import { createElement, forwardRef, useCallback, useRef, useState, useEffect, useMemo } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var getDepthPx = function (depth, depthGap) { return depth * depthGap + "px"; };

var isFunction = function (func) { return func && {}.toString.call(func) === '[object Function]'; };

var defaultProps = {
    childrenKey: 'children',
    classes: {},
    depthGap: 20,
    displayedName: function (node) { return node.data.name; },
    filter: function () { return true; },
    gapMode: 'margin',
    horizontalLineStyles: { stroke: 'black', strokeWidth: 1, strokeDasharray: '1 1' },
    idKey: 'id',
    opened: [],
    verticalLineOffset: 5,
    verticalLineStyles: { stroke: 'black', strokeWidth: 1, strokeDasharray: '1 1' },
    verticalLineTopOffset: 0,
};

var convertObjectToString = function (classes) { return Object
    .keys(classes)
    .filter(function (key) { return !!classes[key]; })
    .reduce(function (classString, item) { return (classString
    ? "" + classString + (item ? " " + item : '')
    : "" + item); }, ''); };
var classnames = function () {
    var classes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        classes[_i] = arguments[_i];
    }
    if (classes[0] && typeof classes[0] === 'string') {
        return classes.join(' ');
    }
    return convertObjectToString(classes[0]);
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".style__hyper-node-arrow-icon___2JsZ4 {\n  transition: transform 0.1s ease;\n  cursor: pointer;\n  margin-right: 4px; }\n  .style__hyper-node-arrow-icon___2JsZ4.style__opened___18SZM {\n    transform: rotate(90deg); }\n";
var styles = {"hyper-node-arrow-icon":"style__hyper-node-arrow-icon___2JsZ4","opened":"style__opened___18SZM","hyperNodeArrowIcon":"style__hyper-node-arrow-icon___2JsZ4"};
styleInject(css);

var Arrow = function (_a) {
    var _b;
    var onClick = _a.onClick, opened = _a.opened;
    return (createElement("svg", { className: classnames((_b = {}, _b[styles.hyperNodeArrowIcon] = true, _b[styles.opened] = !!opened, _b)), xmlns: "http://www.w3.org/2000/svg", width: "10", height: "10", viewBox: "0 0 24 24", onClick: onClick },
        createElement("path", { d: "M5 3l3.057-3 11.943 12-11.943 12-3.057-3 9-9z" })));
};

var css$1 = ".style__loading-spinner___3WGvQ {\n  transform: scale(0.3);\n  animation: style__loading-spinner___3WGvQ 1s linear infinite;\n  margin-right: 4px; }\n\n@keyframes style__loading-spinner___3WGvQ {\n  from {\n    transform: rotate(0deg); }\n  to {\n    transform: rotate(360deg); } }\n";
var styles$1 = {"loading-spinner":"style__loading-spinner___3WGvQ","loadingSpinner":"style__loading-spinner___3WGvQ"};
styleInject(css$1);

var Loader = function () { return (createElement("svg", { className: styles$1.loadingSpinner, xmlns: "http://www.w3.org/2000/svg", width: "10", height: "10", viewBox: "0 0 48 48" },
    createElement("g", { fill: "none" },
        createElement("path", { id: "track", fill: "#C6CCD2", d: "M24,48 C10.745166,48 0,37.254834 0,24 C0,10.745166 10.745166,0 24,0\n        C37.254834,0 48,10.745166 48,24 C48,37.254834 37.254834,48 24,48 Z M24,44\n        C35.045695,44 44,35.045695 44,24 C44,12.954305 35.045695,4 24,4\n        C12.954305,4 4,12.954305 4,24 C4,35.045695 12.954305,44 24,44 Z" }),
        createElement("path", { id: "section", fill: "#3F4850", d: "M24,0 C37.254834,0 48,10.745166 48,24 L44,24 C44,12.954305 35.045695,4 24,4 L24,0 Z" })))); };

var css$2 = ".style__hyper-node___1xbGi {\n  display: flex;\n  align-items: center; }\n";
var styles$2 = {"hyper-node":"style__hyper-node___1xbGi","hyperNode":"style__hyper-node___1xbGi"};
styleInject(css$2);

var DefaultNode = function (_a) {
    var _b = _a.displayedName, displayedName = _b === void 0 ? defaultProps.displayedName : _b, node = _a.node, onSelect = _a.onSelect, onToggle = _a.onToggle;
    return (createElement("div", { className: styles$2.hyperNode, onClick: onSelect },
        (node.hasChildren() || node.options.async)
            && !node.isLoading()
            && (createElement(Arrow, { onClick: onToggle, opened: node.isOpened() && !!node.hasChildren() })),
        node.isLoading() && (createElement(Loader, null)),
        createElement("div", null, isFunction(displayedName) ? displayedName(node) : node.data[displayedName])));
};

var css$3 = ".style__vertical-line___3DAX3 {\n  position: absolute;\n  top: 0;\n  transition: all 0.2s ease; }\n  .style__vertical-line___3DAX3 .style__vertical-svg-line___wptrc {\n    transition: all 0.2s ease; }\n";
var styles$3 = {"vertical-line":"style__vertical-line___3DAX3","vertical-svg-line":"style__vertical-svg-line___wptrc","verticalLine":"style__vertical-line___3DAX3","verticalSvgLine":"style__vertical-svg-line___wptrc"};
styleInject(css$3);

var VerticalLine = function (_a) {
    var depth = _a.depth, depthGap = _a.depthGap, count = _a.count, nodeHeight = _a.nodeHeight, _b = _a.verticalLineOffset, verticalLineOffset = _b === void 0 ? defaultProps.verticalLineOffset : _b, _c = _a.verticalLineStyles, verticalLineStyles = _c === void 0 ? defaultProps.verticalLineStyles : _c, _d = _a.verticalLineTopOffset, verticalLineTopOffset = _d === void 0 ? defaultProps.verticalLineTopOffset : _d;
    return (createElement("div", { className: styles$3.verticalLine, style: {
            left: verticalLineOffset + "px",
            marginLeft: getDepthPx(depth, depthGap),
            top: verticalLineTopOffset + "px"
        } },
        createElement("svg", { style: {
                height: Math.floor(count * nodeHeight - (nodeHeight ? nodeHeight / 2 : 0)) - verticalLineTopOffset + "px",
                width: verticalLineStyles.strokeWidth + "px",
                position: 'absolute',
                transition: 'all 0.2s ease',
            } },
            createElement("line", { x1: "0%", y1: "0%", x2: "0%", y2: "100%", className: styles$3.verticalSvgLine, style: verticalLineStyles }))));
};

var css$4 = ".style__horizontal-line___3_N8i {\n  position: absolute;\n  top: 0;\n  display: flex;\n  align-items: center;\n  height: 100%; }\n";
var styles$4 = {"horizontal-line":"style__horizontal-line___3_N8i","horizontalLine":"style__horizontal-line___3_N8i"};
styleInject(css$4);

var HorizontalLine = function (_a) {
    var depth = _a.depth, depthGap = _a.depthGap, _b = _a.verticalLineOffset, verticalLineOffset = _b === void 0 ? defaultProps.verticalLineOffset : _b, _c = _a.horizontalLineStyles, horizontalLineStyles = _c === void 0 ? defaultProps.horizontalLineStyles : _c, gapMode = _a.gapMode;
    return (createElement("div", { className: styles$4.horizontalLine, style: {
            width: depthGap - verticalLineOffset + "px",
            left: gapMode === 'padding'
                ? verticalLineOffset - depthGap + depth * depthGap + "px"
                : verticalLineOffset - depthGap + "px",
        } },
        createElement("svg", { style: {
                height: horizontalLineStyles.strokeWidth + "px",
                width: '100%',
            } },
            createElement("line", { x1: "0%", y1: "0%", x2: "100%", y2: "0%", style: horizontalLineStyles }))));
};

var css$5 = ".style__drag-zone___2KLU6 {\n  position: absolute;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  left: 0;\n  display: flex;\n  flex-direction: column;\n  visibility: hidden;\n  opacity: 0; }\n  .style__drag-zone___2KLU6.style__dragZoneVisible___xCdOl {\n    visibility: visible;\n    opacity: 1; }\n  .style__drag-zone___2KLU6 .style__drag-zone-item___1d2AK {\n    transition: background-color 0.2s ease;\n    background-color: transparent;\n    width: 100%;\n    height: 100%; }\n    .style__drag-zone___2KLU6 .style__drag-zone-item___1d2AK.style__drag-zone-item-selected___WaRmt {\n      background-color: rgba(179, 255, 192, 0.5); }\n";
var styles$5 = {"drag-zone":"style__drag-zone___2KLU6","dragZoneVisible":"style__dragZoneVisible___xCdOl","drag-zone-item":"style__drag-zone-item___1d2AK","drag-zone-item-selected":"style__drag-zone-item-selected___WaRmt","dragZone":"style__drag-zone___2KLU6","dragZoneItem":"style__drag-zone-item___1d2AK","dragZoneItemSelected":"style__drag-zone-item-selected___WaRmt"};
styleInject(css$5);

var DragZone = function (_a) {
    var _b, _c, _d, _e;
    var depth = _a.depth, depthGap = _a.depthGap, isDragging = _a.isDragging, node = _a.node, onDragEnterAfter = _a.onDragEnterAfter, onDragEnterBefore = _a.onDragEnterBefore, onDragEnterChildren = _a.onDragEnterChildren, onDragLeave = _a.onDragLeave;
    return (createElement("div", { className: classnames((_b = {},
            _b[styles$5.dragZone] = true,
            _b[styles$5.dragZoneVisible] = !!isDragging,
            _b)), style: { marginLeft: getDepthPx(depth, depthGap) }, onDragLeave: onDragLeave },
        createElement("div", { className: classnames((_c = {},
                _c[styles$5.dragZoneItem] = true,
                _c[styles$5.dragZoneItemSelected] = node.getNodeDropContainer() === 'before',
                _c)), onDragEnter: onDragEnterBefore }),
        createElement("div", { className: classnames((_d = {},
                _d[styles$5.dragZoneItem] = true,
                _d[styles$5.dragZoneItemSelected] = node.getNodeDropContainer() === 'children',
                _d)), onDragEnter: onDragEnterChildren }),
        createElement("div", { className: classnames((_e = {},
                _e[styles$5.dragZoneItem] = true,
                _e[styles$5.dragZoneItemSelected] = node.getNodeDropContainer() === 'after',
                _e)), onDragEnter: onDragEnterAfter })));
};

var css$6 = ".style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH {\n  position: relative;\n  transition: all 0.2s ease; }\n  .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__hyper-node-wrapper___348bw {\n    position: relative;\n    transition: all 0.2s ease;\n    opacity: 1; }\n  .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__child___2_fIN {\n    position: relative;\n    transition: all 0.2s ease;\n    transform-origin: top;\n    transform-style: flat;\n    max-height: 9999px;\n    transform: scaleY(1);\n    opacity: 1;\n    visibility: visible; }\n    .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__child___2_fIN.style__collapsed___3-8dN {\n      transition: all 0.2s cubic-bezier(0, 1, 0, 1);\n      transform: scaleY(0);\n      max-height: 0;\n      opacity: 0;\n      visibility: hidden;\n      overflow: hidden; }\n";
var styles$6 = {"hyper-tree-view":"style__hyper-tree-view___3CtKm","hyper-tree-node-wrapper":"style__hyper-tree-node-wrapper___1l1NH","hyper-node-wrapper":"style__hyper-node-wrapper___348bw","child":"style__child___2_fIN","collapsed":"style__collapsed___3-8dN","hyperTreeView":"style__hyper-tree-view___3CtKm","hyperTreeNodeWrapper":"style__hyper-tree-node-wrapper___1l1NH","hyperNodeWrapper":"style__hyper-node-wrapper___348bw"};
styleInject(css$6);

var HyperTreeNode = forwardRef(function (_a, ref) {
    var _b, _c;
    var _d = _a.classes, classes = _d === void 0 ? defaultProps.classes : _d, depth = _a.depth, _e = _a.depthGap, depthGap = _e === void 0 ? defaultProps.depthGap : _e, disableHorizontalLines = _a.disableHorizontalLines, disableLines = _a.disableLines, _f = _a.displayedName, displayedName = _f === void 0 ? defaultProps.displayedName : _f, draggable = _a.draggable, draggableHandlers = _a.draggableHandlers, _g = _a.gapMode, gapMode = _g === void 0 ? defaultProps.gapMode : _g, horizontalLineStyles = _a.horizontalLineStyles, isDragging = _a.isDragging, node = _a.node, renderDragZone = _a.renderDragZone, renderNode = _a.renderNode, setOpen = _a.setOpen, setSelected = _a.setSelected, staticNodeHeight = _a.staticNodeHeight, _h = _a.verticalLineOffset, verticalLineOffset = _h === void 0 ? defaultProps.verticalLineOffset : _h;
    var handleClick = useCallback(function (e) {
        e.stopPropagation();
        if (setOpen) {
            setOpen(node);
        }
    }, [node, setOpen]);
    var handleSelected = useCallback(function () {
        if (setSelected) {
            setSelected(node, !node.isSelected());
        }
    }, [node, setSelected]);
    var additionStyles = {};
    if (staticNodeHeight) {
        additionStyles.height = staticNodeHeight + "px";
    }
    var offsetProp = gapMode === 'padding' ? 'paddingLeft' : 'marginLeft';
    var handleDragEnter = useCallback(function (type) { return draggableHandlers.handleDragEnter(node, type); }, [draggableHandlers, node]);
    return (createElement("div", { draggable: draggable, onDragStart: draggable ? draggableHandlers.handleDragStart : undefined, onDragEnd: draggable ? draggableHandlers.handleDrop(node, node.getNodeDropContainer()) : undefined, className: classnames((_b = {},
            _b[styles$6.hyperNodeWrapper] = true,
            _b[styles$6.selected] = node.isSelected(),
            _b[styles$6.dropContainer] = !!node.getNodeDropContainer(),
            _b[classes.nodeWrapper || ''] = !!classes.nodeWrapper,
            _b[classes.selectedNodeWrapper || ''] = !!(classes.selectedNodeWrapper && node.isSelected()),
            _b)), style: __assign((_c = {}, _c[offsetProp] = getDepthPx(depth, depthGap), _c), additionStyles), ref: ref },
        draggable && renderDragZone
            ? renderDragZone({
                depth: depth,
                depthGap: depthGap,
                isDragging: isDragging,
                node: node,
                onDragEnterAfter: handleDragEnter('after'),
                onDragEnterBefore: handleDragEnter('before'),
                onDragEnterChildren: handleDragEnter('children'),
                onDragLeave: draggableHandlers.handleDragLeave(node),
            }) : draggable && (createElement(DragZone, { depth: depth, depthGap: depthGap, isDragging: isDragging, node: node, onDragEnterAfter: handleDragEnter('after'), onDragEnterBefore: handleDragEnter('before'), onDragEnterChildren: handleDragEnter('children'), onDragLeave: draggableHandlers.handleDragLeave(node) })),
        renderNode
            ? renderNode({
                depth: depth,
                depthGap: depthGap,
                displayedName: displayedName,
                node: node,
                nodeRef: ref,
                offsetProp: offsetProp,
                onSelect: handleSelected,
                onToggle: handleClick,
            }) : (createElement(DefaultNode, { displayedName: displayedName, node: node, onSelect: handleSelected, onToggle: handleClick })),
        !node.options.root && !(disableLines || disableHorizontalLines) && (createElement(HorizontalLine, { depth: depth, depthGap: depthGap, gapMode: gapMode, horizontalLineStyles: horizontalLineStyles, verticalLineOffset: verticalLineOffset }))));
});
var HyperTreeViewChildren = function (_a) {
    var _b;
    var _c = _a.classes, classes = _c === void 0 ? defaultProps.classes : _c, depth = _a.depth, _d = _a.depthGap, depthGap = _d === void 0 ? defaultProps.depthGap : _d, disableLines = _a.disableLines, disableVerticalLines = _a.disableVerticalLines, node = _a.node, nodeHeight = _a.nodeHeight, verticalLineOffset = _a.verticalLineOffset, verticalLineStyles = _a.verticalLineStyles, verticalLineTopOffset = _a.verticalLineTopOffset, props = __rest(_a, ["classes", "depth", "depthGap", "disableLines", "disableVerticalLines", "node", "nodeHeight", "verticalLineOffset", "verticalLineStyles", "verticalLineTopOffset"]);
    if (!node.hasChildren()) {
        return null;
    }
    return (createElement("div", { className: classnames((_b = {},
            _b[styles$6.child] = true,
            _b[styles$6.collapsed] = !node.isOpened(),
            _b[classes.children || ''] = !!classes.children,
            _b[classes.collapsedChildren || ''] = !!(classes.collapsedChildren && !node.isOpened()),
            _b)) },
        createElement(HyperTreeView, __assign({ classes: classes, data: node.getChildren(), depth: depth + 1, depthGap: depthGap, disableLines: disableLines, disableVerticalLines: disableVerticalLines, nodeHeight: nodeHeight, verticalLineOffset: verticalLineOffset, verticalLineStyles: verticalLineStyles, verticalLineTopOffset: verticalLineTopOffset }, props)),
        !(disableLines || disableVerticalLines) && (createElement(VerticalLine, { count: node.options.currentChilrenCount, depth: depth, depthGap: depthGap, nodeHeight: nodeHeight, verticalLineOffset: verticalLineOffset, verticalLineStyles: verticalLineStyles, verticalLineTopOffset: verticalLineTopOffset }))));
};
var HyperTreeView = forwardRef(function (_a, ref) {
    var _b = _a.classes, classes = _b === void 0 ? defaultProps.classes : _b, data = _a.data, _c = _a.depth, depth = _c === void 0 ? 0 : _c, props = __rest(_a, ["classes", "data", "depth"]);
    return (createElement("div", { className: classnames(styles$6.hyperTreeView, classes.level) }, data.map(function (currentNode) { return (createElement("div", { key: currentNode.id, className: classnames(styles$6.hyperTreeNodeWrapper, classes.parentChildren) },
        createElement(HyperTreeNode, __assign({ classes: classes, depth: depth, node: currentNode, ref: ref }, props)),
        createElement(HyperTreeViewChildren, __assign({ classes: classes, depth: depth, node: currentNode }, props)))); })));
});
var Tree = function (_a) {
    var _b = _a.classes, classes = _b === void 0 ? defaultProps.classes : _b, staticNodeHeight = _a.staticNodeHeight, props = __rest(_a, ["classes", "staticNodeHeight"]);
    var nodeRef = useRef(null);
    var _c = useState(staticNodeHeight || 0), height = _c[0], setHeight = _c[1];
    useEffect(function () {
        if (nodeRef.current && !height && !staticNodeHeight) {
            setHeight(nodeRef.current.clientHeight);
        }
    }, [height, staticNodeHeight]);
    return (createElement("div", { className: classes.treeWrapper },
        createElement(HyperTreeView, __assign({ classes: classes, depth: 0, nodeHeight: height, ref: nodeRef, staticNodeHeight: staticNodeHeight }, props))));
};

var getRandom = function () { return Math.random().toString(16).slice(-4); };
var uuid = function () { return getRandom() + "-" + getRandom() + "-" + getRandom() + "-" + getRandom() + getRandom(); };

var TreeNode = /** @class */ (function () {
    function TreeNode(data, options, children) {
        this.data = data;
        this.id = data[options.idKey] || uuid();
        this.options = options;
        this.children = children;
    }
    TreeNode.prototype.getData = function () {
        return this.data;
    };
    TreeNode.prototype.setData = function (data) {
        this.data = data;
    };
    TreeNode.prototype.getChildren = function () {
        return this.children || [];
    };
    TreeNode.prototype.setNodeChildren = function (children, type, reset) {
        if (!this.hasChildren() || reset) {
            this.setChildren(this.setNodesParent(children, this));
            return this.getChildren();
        }
        if (type === 'first') {
            this.setChildren(this.setNodesParent(__spreadArrays(children, (this.children || [])), this));
            return this.getChildren();
        }
        this.setChildren(this.setNodesParent(__spreadArrays((this.children || []), children), this));
        return this.getChildren();
    };
    TreeNode.prototype.removeChild = function (node) {
        this.setChildren(this.getChildren().filter(function (child) { return child.id !== node.id; }));
    };
    TreeNode.prototype.setChildren = function (children) {
        this.children = this.setNodesParent(children, this);
    };
    TreeNode.prototype.hasChildren = function () {
        return this.children && this.children.length !== 0;
    };
    TreeNode.prototype.getParent = function () {
        return this.options.parent;
    };
    TreeNode.prototype.setParent = function (parent) {
        this.options.parent = parent;
    };
    TreeNode.prototype.setNodesParent = function (nodes, parent) {
        return nodes.map(function (node) {
            node.setParent(parent);
            return node;
        });
    };
    TreeNode.prototype.setChildrenCount = function (count) {
        this.options.childrenCount = count;
    };
    TreeNode.prototype.getFirstChild = function () {
        return this.getChildren()[0] || null;
    };
    TreeNode.prototype.getLastChild = function () {
        return this.children && this.children.length !== 0 ? this.children[this.children.length - 1] : null;
    };
    TreeNode.prototype.setOpened = function (opened) {
        this.options.opened = opened || false;
    };
    TreeNode.prototype.isOpened = function () {
        return this.options.opened;
    };
    TreeNode.prototype.setLoading = function (loading) {
        this.options.loading = loading || false;
    };
    TreeNode.prototype.isLoading = function () {
        return this.options.loading;
    };
    TreeNode.prototype.setSelected = function (selected) {
        this.options.selected = selected || false;
    };
    TreeNode.prototype.isSelected = function () {
        return this.options.selected;
    };
    TreeNode.prototype.setPath = function (parentPath) {
        this.options.path = parentPath + "/" + this.id;
    };
    TreeNode.prototype.getPath = function (array) {
        if (array) {
            return this.options.path.split('/').filter(function (id) { return !!id; });
        }
        return this.options.path;
    };
    TreeNode.prototype.setNodeDropContainer = function (dropContainer) {
        if (dropContainer === void 0) { dropContainer = false; }
        this.options.dropContainer = dropContainer;
    };
    TreeNode.prototype.getNodeDropContainer = function () {
        return this.options.dropContainer;
    };
    return TreeNode;
}());
var TreeView = /** @class */ (function () {
    function TreeView(id, data, options) {
        this.id = id;
        this.data = data;
        this.options = options;
        this.enhancedData = [];
        this.hash = uuid();
        this.ids = [];
        if (options.enhance) {
            this.enhance();
        }
    }
    TreeView.prototype.on = function (callback) {
        callback();
    };
    TreeView.prototype.sort = function (tree) {
        tree.sort(this.options.sort);
        return tree;
    };
    TreeView.prototype.staticEnhance = function (data, node) {
        var currentNode;
        if (node instanceof TreeNode) {
            currentNode = node;
        }
        else if (node) {
            currentNode = this.getNodeById(node);
        }
        return this._staticEnhance(data, currentNode);
    };
    TreeView.prototype.enhance = function (data) {
        this.enhancedData = this._enhance(data || this.data, this.options);
        return this.enhancedData;
    };
    TreeView.prototype.enhanceNodes = function (shallow) {
        var _this = this;
        if (shallow === void 0) { shallow = false; }
        if (shallow) {
            this.enhancedData = this._staticShallowEnahance(this.enhancedData);
        }
        this.traverse(function (node) {
            _this._calculateChildrenCount(node);
        }, false);
    };
    TreeView.prototype.flat = function (raw) {
        var tree = raw ? this.data : this.enhancedData;
        this.flatData = this._flat(tree);
        this.ids = this.flatData.map(function (item) { return item.id; });
        return this.flatData;
    };
    TreeView.prototype.unselectAll = function () {
        this.traverse(function (node) {
            node.setSelected(false);
        });
    };
    TreeView.prototype.addChildren = function (parentId, data, insertType) {
        if (insertType === void 0) { insertType = 'first'; }
        this._addChildren(this.data, parentId, data, insertType);
    };
    TreeView.prototype.addSibling = function (nodeId, data, insertType) {
        if (insertType === void 0) { insertType = 'after'; }
        this._addSibling(this.data, nodeId, data, insertType);
    };
    TreeView.prototype.remove = function (nodeId, leaveChildren) {
        this._remove(this.data, null, nodeId, leaveChildren);
    };
    TreeView.prototype.traverseRaw = function (callback, deep) {
        if (deep === void 0) { deep = true; }
        if (deep) {
            this._traverseDeep(this.data, callback);
        }
        else {
            this._traverseBreadth(this.data, callback);
        }
    };
    TreeView.prototype.traverse = function (callback, deep) {
        if (deep === void 0) { deep = true; }
        if (deep) {
            this._traverseDeep(this.enhancedData, callback);
        }
        else {
            this._traverseBreadth(this.enhancedData, callback);
        }
    };
    TreeView.prototype.getNodeById = function (id) {
        return this._getNodeById(id, this.enhancedData);
    };
    TreeView.prototype._addChildren = function (data, parentId, node, insertType) {
        for (var i = 0; i < data.length; i += 1) {
            if (data[i][this.options.idKey] === parentId) {
                switch (insertType) {
                    case 'first': {
                        data[i][this.options.childrenKey] = __spreadArrays(node, (data[i][this.options.childrenKey] || []));
                        return;
                    }
                    case 'last': {
                        data[i][this.options.childrenKey] = __spreadArrays((data[i][this.options.childrenKey] || []), node);
                        return;
                    }
                    default: return;
                }
            }
            else if (data[i][this.options.childrenKey] && data[i][this.options.childrenKey].length !== 0) {
                this._addChildren(data[i][this.options.childrenKey], parentId, node, insertType);
                return;
            }
            return;
        }
    };
    TreeView.prototype._addSibling = function (data, nodeId, node, insertType) {
        var _this = this;
        for (var i = 0; i < data.length; i += 1) {
            if (data[i][this.options.childrenKey] && data[i][this.options.childrenKey].length !== 0) {
                var index = data[i][this.options.childrenKey].findIndex(function (child) { return child[_this.options.idKey] === nodeId; });
                if (index > -1) {
                    data[i][this.options.childrenKey].splice(insertType === 'before' ? index : index + 1, 0, node);
                    return;
                }
                this._addSibling(data[i][this.options.childrenKey], nodeId, node, insertType);
            }
            return;
        }
    };
    TreeView.prototype._flat = function (tree) {
        var _this = this;
        var result = [];
        tree.forEach(function (child) {
            var children = child.children, childData = __rest(child, ["children"]);
            result.push(childData);
            if (children && children.length !== 0) {
                result.push.apply(result, _this._flat(children));
            }
        });
        return result;
    };
    TreeView.prototype._enhance = function (tree, options, parentNode) {
        var _this = this;
        var result = [];
        this.sort(tree).filter(this.options.filter).forEach(function (child) {
            var filteredChildren = child[_this.options.childrenKey]
                && child[_this.options.childrenKey].length !== 0
                ? child[_this.options.childrenKey].filter(_this.options.filter)
                : [];
            var newChild = new TreeNode(child, {
                opened: !!options.defaultOpened && !child.getChildren,
                selected: false,
                root: !parentNode,
                leaf: !filteredChildren,
                hasChildren: filteredChildren.length !== 0,
                async: !!child.getChildren,
                parent: parentNode,
                childrenCount: 0,
                currentChilrenCount: 0,
                loading: false,
                idKey: _this.options.idKey,
                childrenKey: _this.options.childrenKey,
                path: '',
                dropContainer: false,
            });
            newChild.setPath(parentNode ? parentNode.options.path : '');
            var children = [];
            if (filteredChildren.length !== 0) {
                children = _this._enhance(_this.sort(filteredChildren), options, newChild);
            }
            newChild.setChildren(children);
            _this._calculateChildrenCount(newChild);
            result.push(newChild);
        });
        return result;
    };
    TreeView.prototype._staticEnhance = function (tree, parentNode) {
        var _this = this;
        var result = [];
        tree.forEach(function (child) {
            var curentChildren = child[_this.options.childrenKey] || [];
            var newChild = new TreeNode(child, {
                opened: false,
                selected: false,
                root: !parentNode,
                leaf: curentChildren.length === 0,
                hasChildren: curentChildren.length !== 0,
                async: !!child.getChildren,
                parent: parentNode,
                childrenCount: 0,
                currentChilrenCount: 0,
                loading: false,
                idKey: _this.options.idKey,
                childrenKey: _this.options.childrenKey,
                path: '',
                dropContainer: false,
            });
            newChild.setPath(parentNode ? parentNode.options.path : '');
            var children = [];
            if (curentChildren.length !== 0) {
                children = _this._staticEnhance(curentChildren, newChild);
            }
            newChild.setChildren(children);
            _this._calculateChildrenCount(newChild);
            result.push(newChild);
        });
        return result;
    };
    TreeView.prototype._staticShallowEnahance = function (tree, parentNode) {
        var _this = this;
        var result = [];
        tree.forEach(function (child) {
            var curentChildren = child[_this.options.childrenKey] || [];
            child.options.parent = parentNode;
            child.setPath(parentNode ? parentNode.options.path : '');
            var children = [];
            if (curentChildren.length !== 0) {
                children = _this._staticShallowEnahance(curentChildren, child);
            }
            child.setChildren(children);
            _this._calculateChildrenCount(child);
            result.push(child);
        });
        return result;
    };
    TreeView.prototype._calculateChildrenCount = function (child) {
        var childrenCount = child.getChildren().reduce(function (acc, c) {
            if (c.isOpened()) {
                acc += c.options.childrenCount;
                return acc;
            }
            return acc;
        }, 0);
        if (child.isOpened()) {
            child.options.childrenCount = child.getChildren().length + childrenCount;
        }
        else {
            child.options.childrenCount = child.getChildren().length;
        }
        var lastChild = child.getLastChild();
        if (lastChild && lastChild.isOpened()) {
            var globalCount = child.options.childrenCount;
            child.options.currentChilrenCount = globalCount - lastChild.options.childrenCount;
        }
        else {
            child.options.currentChilrenCount = child.options.childrenCount;
        }
    };
    TreeView.prototype._remove = function (data, parentNode, nodeId, leaveChildren) {
        for (var i = 0; i < data.length; i += 1) {
            if (data[i][this.options.idKey] === nodeId) {
                var nodeToRemove = data[i];
                data.splice(i, 1);
                if (parentNode
                    && nodeToRemove[this.options.childrenKey]
                    && nodeToRemove[this.options.childrenKey].length !== 0
                    && leaveChildren) {
                    parentNode[this.options.childrenKey] = leaveChildren === 'start'
                        ? __spreadArrays(nodeToRemove[this.options.childrenKey], (parentNode[this.options.childrenKey] || [])) : __spreadArrays((parentNode[this.options.childrenKey] || []), nodeToRemove[this.options.childrenKey]);
                }
                return;
            }
            if (data[i][this.options.childrenKey] && data[i][this.options.childrenKey].length !== 0) {
                this._remove(data[i][this.options.childrenKey], data[i], nodeId, leaveChildren);
            }
        }
    };
    TreeView.prototype._getNodeById = function (id, tree) {
        for (var i = 0; i < tree.length; i += 1) {
            if (tree[i][this.options.idKey] === id) {
                return tree[i];
            }
            if (tree[i][this.options.childrenKey] && tree[i][this.options.childrenKey].length !== 0) {
                var node = this._getNodeById(id, tree[i][this.options.childrenKey]);
                if (node) {
                    return node;
                }
            }
        }
        return null;
    };
    TreeView.prototype._traverseDeep = function (tree, callback, depth) {
        var _this = this;
        if (depth === void 0) { depth = 0; }
        tree.forEach(function (node, index) {
            callback(node, index, depth);
            if (node.children && node.children.length !== 0) {
                _this._traverseDeep(node.children, callback, depth + 1);
            }
        });
    };
    TreeView.prototype._traverseBreadth = function (tree, callback, depth) {
        var _this = this;
        if (depth === void 0) { depth = 0; }
        tree.forEach(function (node, index) {
            if (node.children && node.children.length !== 0) {
                _this._traverseDeep(node.children, callback, depth + 1);
            }
            callback(node, index, depth);
        });
    };
    return TreeView;
}());

var TreeHandlers = /** @class */ (function () {
    function TreeHandlers() {
        this.trees = {};
    }
    TreeHandlers.prototype.getIds = function () {
        return Object.keys(this.trees);
    };
    TreeHandlers.prototype.safeUpdate = function (id, tree) {
        this.trees[id] = {
            instance: tree,
            handlers: {},
        };
        return this;
    };
    TreeHandlers.prototype.remove = function (id) {
        var _a = this.trees, _b = id, _ = _a[_b], trees = __rest(_a, [typeof _b === "symbol" ? _b : _b + ""]);
        this.trees = trees;
        return this;
    };
    TreeHandlers.prototype.safeUpdateHandler = function (treeId, handlerName, handler) {
        var _a;
        if (this.trees[treeId]) {
            this.trees[treeId].handlers = __assign(__assign({}, this.trees[treeId].handlers), (_a = {}, _a[handlerName] = handler, _a));
        }
        return this;
    };
    TreeHandlers.prototype.removeHandler = function (treeId, handlerName) {
        if (this.trees[treeId]) {
            var _a = this.trees[treeId].handlers, _b = handlerName, _ = _a[_b], handlers = __rest(_a, [typeof _b === "symbol" ? _b : _b + ""]);
            this.trees[treeId].handlers = handlers;
        }
        return this;
    };
    return TreeHandlers;
}());
var treeHandlers = new TreeHandlers();

var defaultOptions = {
    filter: function () { return true; },
    sort: function () { return 0; },
};
var useForceUpdate = function () {
    var _a = useState(Object.create(null)), dispatch = _a[1];
    var memoizedDispatch = useCallback(function (callback) {
        if (callback) {
            callback();
        }
        dispatch(Object.create(null));
    }, [dispatch]);
    return memoizedDispatch;
};
var useTreeState = function (_a) {
    var id = _a.id, data = _a.data, filter = _a.filter, sort = _a.sort, defaultOpened = _a.defaultOpened, multipleSelect = _a.multipleSelect, _b = _a.idKey, idKey = _b === void 0 ? defaultProps.idKey : _b, _c = _a.childrenKey, childrenKey = _c === void 0 ? defaultProps.childrenKey : _c;
    var forceUpdate = useForceUpdate();
    var _d = useState(null), dropNodeId = _d[0], setDropNodeId = _d[1];
    var _e = useState(false), isDragging = _e[0], setDragging = _e[1];
    var _f = useState(false), dropType = _f[0], setDropType = _f[1];
    var treeView = useMemo(function () { return new TreeView(id, Array.isArray(data) ? data : [data], {
        idKey: idKey,
        childrenKey: childrenKey,
        defaultOpened: defaultOpened,
        enhance: true,
        filter: filter || defaultOptions.filter,
        sort: sort || defaultOptions.sort,
    }); }, [data, defaultOpened, filter, id, sort, idKey, childrenKey]);
    var setLoading = useCallback(function (node, loading) {
        if (node instanceof TreeNode) {
            node.setLoading(loading);
            forceUpdate();
        }
        else {
            var foundNode = treeView.getNodeById(node);
            if (foundNode) {
                foundNode.setLoading(loading);
                forceUpdate();
            }
        }
    }, [forceUpdate, treeView]);
    var setSelected = useCallback(function (node, selected) {
        if (!multipleSelect && selected) {
            treeView.unselectAll();
        }
        if (node instanceof TreeNode) {
            node.setSelected(selected);
            forceUpdate();
        }
        else {
            var foundNode = treeView.getNodeById(node);
            if (foundNode) {
                foundNode.setSelected(selected);
                forceUpdate();
            }
        }
    }, [forceUpdate, treeView, multipleSelect]);
    var setDragContainer = useCallback(function (node, dragContainer) {
        if (node instanceof TreeNode) {
            node.setNodeDropContainer(dragContainer);
            forceUpdate();
        }
        else {
            var foundNode = treeView.getNodeById(node);
            if (foundNode) {
                foundNode.setNodeDropContainer(dragContainer);
                forceUpdate();
            }
        }
    }, [forceUpdate, treeView]);
    var setChildren = useCallback(function (parent, children, type, reset) {
        if (parent instanceof TreeNode) {
            parent.setNodeChildren(children, type, reset);
            treeView.enhanceNodes();
            forceUpdate();
        }
        else {
            var foundParent = treeView.getNodeById(parent);
            if (foundParent) {
                foundParent.setNodeChildren(children, type, reset);
                treeView.enhanceNodes();
                forceUpdate();
            }
        }
    }, [treeView, forceUpdate]);
    var setSiblings = useCallback(function (node, siblings, type) {
        var _a;
        var targetNode = null;
        if (node instanceof TreeNode) {
            targetNode = node;
        }
        else {
            targetNode = treeView.getNodeById(node);
        }
        if (targetNode && targetNode.options.parent) {
            var parentChildren = __spreadArrays(targetNode.options.parent.getChildren());
            var nodeIndex = parentChildren.findIndex(function (child) { return targetNode && child.id === targetNode.id; });
            if (nodeIndex !== -1) {
                var startIndex = type === 'before' ? nodeIndex : nodeIndex + 1;
                parentChildren.splice.apply(parentChildren, __spreadArrays([startIndex, 0], siblings));
                targetNode.options.parent.setNodeChildren(parentChildren, 'first', true);
                treeView.enhanceNodes(true);
                forceUpdate();
            }
        }
        if (targetNode && targetNode.options.root) {
            var nodeIndex = treeView.enhancedData.findIndex(function (child) { return targetNode && targetNode.id === child.id; });
            if (nodeIndex !== -1) {
                var startIndex = type === 'before' ? nodeIndex : nodeIndex + 1;
                var mappedSiblings = siblings.map(function (sibling) {
                    sibling.options.parent = undefined;
                    sibling.options.root = true;
                    sibling.options.leaf = false;
                    return sibling;
                });
                (_a = treeView.enhancedData).splice.apply(_a, __spreadArrays([startIndex, 0], mappedSiblings));
                treeView.enhanceNodes(true);
                forceUpdate();
            }
        }
    }, [treeView, forceUpdate]);
    var setRawChildren = useCallback(function (parent, children, type, reset) {
        setChildren(parent, treeView.staticEnhance(children, parent), type, reset);
    }, [setChildren, treeView]);
    var setOpen = useCallback(function (node) {
        var currentNode;
        if (node instanceof TreeNode) {
            currentNode = node;
        }
        else {
            currentNode = treeView.getNodeById(node);
        }
        if (!currentNode) {
            return;
        }
        if (currentNode.data.getChildren && isFunction(currentNode.data.getChildren) && !currentNode.isOpened()) {
            setLoading(currentNode, true);
            var result = currentNode.data.getChildren({ node: node });
            if (result.then) {
                result.then(function (asyncData) {
                    setLoading(node, false);
                    currentNode.setOpened(true);
                    setRawChildren(node, asyncData, 'last', true);
                });
            }
        }
        else {
            currentNode.setOpened(!currentNode.isOpened());
            treeView.enhanceNodes();
        }
        forceUpdate(function () {
            treeView.enhanceNodes();
        });
    }, [forceUpdate, treeView, setLoading, setRawChildren]);
    var handleDragStart = useCallback(function (e) {
        e.stopPropagation();
        setDragging(true);
    }, []);
    var handleDragEnter = useCallback(function (node, type) { return function (e) {
        e.stopPropagation();
        e.preventDefault();
        setDragContainer(node, type);
        setDropNodeId(node.id);
        setDropType(type);
    }; }, [setDragContainer]);
    var handleDragLeave = useCallback(function (node) { return function (e) {
        e.stopPropagation();
        e.preventDefault();
        if (node.id !== dropNodeId) {
            setDragContainer(node, false);
        }
    }; }, [setDragContainer, dropNodeId]);
    var handleDrop = useCallback(function (sourceNode) { return function (e) {
        e.stopPropagation();
        e.preventDefault();
        setDragging(false);
        if (dropNodeId) {
            setDragContainer(dropNodeId, false);
        }
        if (dropNodeId && dropNodeId !== sourceNode.id) {
            var targetNode = treeView.getNodeById(dropNodeId);
            if (sourceNode) {
                if (sourceNode.options.parent) {
                    sourceNode.options.parent.removeChild(sourceNode);
                }
                else {
                    treeView.enhancedData = treeView.enhancedData.filter(function (child) { return child.id !== sourceNode.id; });
                }
                if (dropType === 'children') {
                    sourceNode.options.parent = targetNode;
                    sourceNode.options.root = false;
                    sourceNode.options.leaf = true;
                    targetNode.setNodeChildren([sourceNode], 'last');
                    treeView.enhanceNodes(true);
                    forceUpdate();
                }
                else if (dropType === 'after' || dropType === 'before') {
                    setSiblings(targetNode, [sourceNode], dropType);
                }
                setDropNodeId(null);
            }
        }
    }; }, [dropNodeId, forceUpdate, treeView, setDragContainer, setSiblings, dropType]);
    treeHandlers
        .safeUpdate(id, treeView)
        .safeUpdateHandler(id, 'rerender', forceUpdate)
        .safeUpdateHandler(id, 'setOpen', setOpen)
        .safeUpdateHandler(id, 'setLoading', setLoading)
        .safeUpdateHandler(id, 'setSelected', setSelected)
        .safeUpdateHandler(id, 'setRawChildren', setRawChildren)
        .safeUpdateHandler(id, 'setChildren', setChildren)
        .safeUpdateHandler(id, 'setSiblings', setSiblings);
    var handlers = useMemo(function () { return ({
        setChildren: setChildren,
        setLoading: setLoading,
        setOpen: setOpen,
        setRawChildren: setRawChildren,
        setSelected: setSelected,
        setSiblings: setSiblings,
        draggableHandlers: {
            handleDragStart: handleDragStart,
            handleDragEnter: handleDragEnter,
            handleDragLeave: handleDragLeave,
            handleDrop: handleDrop,
        },
    }); }, [
        handleDragEnter,
        handleDragLeave,
        handleDragStart,
        handleDrop,
        setChildren,
        setLoading,
        setOpen,
        setRawChildren,
        setSelected,
        setSiblings,
    ]);
    return {
        instance: treeView,
        handlers: handlers,
        required: {
            isDragging: isDragging,
            data: treeView.enhancedData,
        },
    };
};

export default Tree;
export { Arrow, DefaultNode, DragZone, HorizontalLine, Loader, Tree, VerticalLine, treeHandlers, useTreeState };
//# sourceMappingURL=index.es.js.map
